@@ -19,15 +19,10 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.c,v 1.34 2004/06/21 00:22:38 evertonm Exp $
+  $Id: ruli_res.c,v 1.35 2004/08/25 19:04:23 evertonm Exp $
   */
 
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE /* for GNU strncasecmp() */
-#endif
-
-
 #include <stdio.h>    /* FIXME: remove me [used for fprintf() debug] */
 
 #include <strings.h>
@@ -94,13 +89,6 @@ static const ruli_qclass_pair_t ruli_qclass_map[] =
   { 0,    0 }
 };
 
-
-#ifdef _RULI_SOLARIS_
-/* should be in <strings.h> */
-extern int strcasecmp(const char *s1, const char *s2);
-#endif
-
-
 int ruli_get_qtype_code(const char *qt_name) 
 {
   int i;
