@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_smtp.c,v 1.14 2004/06/23 22:10:55 evertonm Exp $
+  $Id: ruli_smtp.c,v 1.17 2004/06/25 23:19:11 evertonm Exp $
   */
 
 
@@ -34,15 +34,15 @@ Boston, MA 02111-1307, USA.
 #include <ruli_host.h>
 
 
-static void f_query_done(ruli_srv_t *srv_qry) {
-  assert(srv_qry->fall_query);
+static void _m_query_done(ruli_srv_t *srv_qry, ruli_res_query_t *mx_qry) {
+  assert(mx_qry);
   
-  ruli_res_query_delete(srv_qry->fall_query);
-  ruli_free(srv_qry->fall_query);
-  srv_qry->fall_query = 0;
+  ruli_res_query_delete(mx_qry);
+  ruli_free(mx_qry);
 }
 
-static void *fall_query_done(ruli_srv_t *srv_qry, int srv_result_code)
+static void *mx_query_done(ruli_srv_t *srv_qry, int srv_result_code,
+                           ruli_res_query_t *mx_qry)
 {
 #ifdef RULI_SMTP_DEBUG
       fprintf(stderr, 
@@ -51,12 +51,12 @@ static void *fall_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 	      srv_result_code ? "FAIL" : "SUCCESS", srv_result_code);
 #endif
 
-  f_query_done(srv_qry);
+  _m_query_done(srv_qry, mx_qry);
 
   return _ruli_srv_query_done(srv_qry, srv_result_code);
 }
 
-static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
+static void *on_mx_answer(ruli_res_query_t *mx_qry, void *qry_arg)
 {
   ruli_srv_t *srv_qry           = (ruli_srv_t *) qry_arg;
   int        prev_srv_list_size = ruli_list_size(&srv_qry->answer_srv_list);
@@ -69,8 +69,8 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 
     result = ruli_dname_decode(txt_dname_buf, RULI_LIMIT_DNAME_TEXT_BUFSZ,
 			       &txt_dname_len, 
-			       (const char *) qry->full_dname, 
-			       qry->full_dname_len);
+			       (const char *) mx_qry->full_dname, 
+			       mx_qry->full_dname_len);
     assert(!result);
 
     fprintf(stderr, 
@@ -81,37 +81,35 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
     fprintf(stderr, 
 	    "DEBUG: %s: %s(): id=%d answer_code=%d rcode=%d\n",
 	    __FILE__, __PRETTY_FUNCTION__,
-	    qry->query_id, qry->answer_code, qry->answer_header.rcode);
+	    mx_qry->query_id, mx_qry->answer_code, mx_qry->answer_header.rcode);
   }
 #endif
 
-  assert(qry->answer_code != RULI_SRV_CODE_VOID);
-  assert(srv_qry->fall_query);
-  assert(srv_qry->fall_query == qry);
+  assert(mx_qry->answer_code != RULI_SRV_CODE_VOID);
   assert(!prev_srv_list_size);
 
   /*
    * Query failed?
    */
-  if (qry->answer_code) {
-    srv_qry->f_query_rcode = RULI_RCODE_VOID;
+  if (mx_qry->answer_code) {
+    srv_qry->last_rcode = RULI_RCODE_VOID;
 
-    if (qry->answer_code == RULI_CODE_TIMEOUT)
-      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_ALARM);
+    if (mx_qry->answer_code == RULI_CODE_TIMEOUT)
+      return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_ALARM, mx_qry);
 
-    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_QUERY);
+    return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_QUERY, mx_qry);
   }
 
   /*
    * Bad RCODE ?
    */
   {
-    ruli_uint16_t rcode = qry->answer_header.rcode;
+    ruli_uint16_t rcode = mx_qry->answer_header.rcode;
 
-    srv_qry->f_query_rcode = rcode;
+    srv_qry->last_rcode = rcode;
 
     if (rcode != RULI_RCODE_NOERROR)
-      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_RCODE);
+      return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_RCODE, mx_qry);
   }
 
   /*
@@ -124,26 +122,26 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
     ruli_parse_t parse;
 
     if (ruli_parse_new(&parse))
-      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+      return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER, mx_qry);
 
     {
       int result;
 
       assert(sizeof(ruli_uint8_t) == sizeof(char));
 
-      result = ruli_parse_message(&parse, &qry->answer_header,
-				  (ruli_uint8_t *) qry->answer_buf,
-                                  qry->answer_msg_len);
+      result = ruli_parse_message(&parse, &mx_qry->answer_header,
+				  (ruli_uint8_t *) mx_qry->answer_buf,
+                                  mx_qry->answer_msg_len);
       if (result) {
 	ruli_parse_delete(&parse);
-	return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_PARSE);
+	return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_PARSE, mx_qry);
       }
     }
 
     assert(sizeof(ruli_uint8_t) == sizeof(char));
 
-    wanted_owner     = (ruli_uint8_t *) qry->full_dname;
-    wanted_owner_len = qry->full_dname_len;
+    wanted_owner     = (ruli_uint8_t *) mx_qry->full_dname;
+    wanted_owner_len = mx_qry->full_dname_len;
 
     /*
      * Search for MX records in answer section
@@ -191,8 +189,8 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	 */
 
         if (ruli_dname_compare(rr->owner,
-                               (ruli_uint8_t *) qry->answer_buf,
-                               qry->answer_msg_len,
+                               (ruli_uint8_t *) mx_qry->answer_buf,
+                               mx_qry->answer_msg_len,
                                wanted_owner,
                                wanted_owner_len))
           continue;
@@ -209,10 +207,10 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	  ruli_srv_entry_t *srv_entry;
 
 	  if (ruli_parse_rr_mx(&mx_rdata, rr->rdata, rr->rdlength,
-			       (ruli_uint8_t *) qry->answer_buf,
-			       qry->answer_msg_len)) {
+			       (ruli_uint8_t *) mx_qry->answer_buf,
+			       mx_qry->answer_msg_len)) {
 	    ruli_parse_delete(&parse);
-	    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_PARSE);
+	    return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_PARSE, mx_qry);
 	  }
 
 	  /* 
@@ -227,7 +225,7 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 					      srv_qry->srv_fallback_port);
 	  if (!srv_entry) {
 	    ruli_parse_delete(&parse);
-	    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+	    return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER, mx_qry);
 	  }
 
 	  /*
@@ -248,8 +246,8 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	     */
 
 	    if (ruli_dname_compare(ad_rr->owner,
-				   (ruli_uint8_t *) qry->answer_buf,
-				   qry->answer_msg_len,
+				   (ruli_uint8_t *) mx_qry->answer_buf,
+				   mx_qry->answer_msg_len,
 				   mx_rdata.target,
 				   mx_rdata.target_len))
 	      continue;
@@ -262,14 +260,14 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	    addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
 	    if (!addr) {
 	      ruli_parse_delete(&parse);
-	      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+	      return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER, mx_qry);
 	    }
 	    
 	    /* Save space */
 	    if (ruli_list_push(&srv_entry->addr_list, addr)) {
 	      ruli_free(addr); 
 	      ruli_parse_delete(&parse);
-	      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+	      return mx_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER, mx_qry);
 	    }
 	    
 	    /* Write address into space */
@@ -313,18 +311,14 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	  ruli_parse_delete(&parse);
 
 	  /* Dispose current IN MX fallback query */
-	  f_query_done(srv_qry);
+	  _m_query_done(srv_qry, mx_qry);
 
 	  /*
 	   * Launch queries to fill missing addresses, if any
 	   */
 
-	  assert(!srv_qry->walk_query);
-	  assert(srv_qry->walk_index == -1);
-	  assert(srv_qry->walk_qtype == -1);
-	 
-	  srv_qry->walk_index = 0;
-	  srv_qry->walk_qtype = _ruli_reset_qtype(srv_qry->srv_options);
+	  assert(srv_qry->under.walk_index == -1);
+	  srv_qry->under.walk_index = 0;
 	  
 	  return _ruli_srv_answer_walk(srv_qry);
 	}
@@ -346,7 +340,7 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
   
   /* Dispose failed, current IN MX fallback query,
    that is replaced by IN A fallback query below */
-  f_query_done(srv_qry);
+  _m_query_done(srv_qry, mx_qry);
 
   /* Try default IN A fallback query */
   return _ruli_srv_answer_fallback_addr(srv_qry);
@@ -357,6 +351,8 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
  */
 static void *srv_answer_fallback_smtp(ruli_srv_t *srv_qry)
 {
+  ruli_res_query_t *mx_qry;
+
 #ifdef RULI_SMTP_DEBUG
   {
     char txt_dname_buf[RULI_LIMIT_DNAME_TEXT_BUFSZ];
@@ -375,33 +371,31 @@ static void *srv_answer_fallback_smtp(ruli_srv_t *srv_qry)
   }
 #endif
 
-  assert(!srv_qry->fall_query);
   assert(!ruli_list_size(&srv_qry->answer_srv_list));
 
   /*
    * Allocate space for fallback query
    */
-  srv_qry->fall_query = (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
-  if (!srv_qry->fall_query)
+  mx_qry = (ruli_res_query_t *) ruli_malloc(sizeof(*mx_qry));
+  if (!mx_qry)
     return _ruli_srv_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
 
   /*
    * Initialize fallback query arguments
    */
-  srv_qry->fall_query->q_on_answer     = on_mx_answer;
-  srv_qry->fall_query->q_on_answer_arg = srv_qry;
-  srv_qry->fall_query->q_domain        = srv_qry->srv_domain;
-  srv_qry->fall_query->q_domain_len    = srv_qry->srv_domain_len;
-  srv_qry->fall_query->q_class         = RULI_RR_CLASS_IN;
-  srv_qry->fall_query->q_type          = RULI_RR_TYPE_MX;
-  srv_qry->fall_query->q_options       = srv_qry->srv_options;
+  mx_qry->q_on_answer     = on_mx_answer;
+  mx_qry->q_on_answer_arg = srv_qry;
+  mx_qry->q_domain        = srv_qry->srv_domain;
+  mx_qry->q_domain_len    = srv_qry->srv_domain_len;
+  mx_qry->q_class         = RULI_RR_CLASS_IN;
+  mx_qry->q_type          = RULI_RR_TYPE_MX;
+  mx_qry->q_options       = srv_qry->srv_options;
 
   /*
    * Submit fallback query
    */
-  if (ruli_res_query_submit(srv_qry->srv_resolver, srv_qry->fall_query)) {
-    ruli_free(srv_qry->fall_query);
-    srv_qry->fall_query = 0;
+  if (ruli_res_query_submit(srv_qry->srv_resolver, mx_qry)) {
+    ruli_free(mx_qry);
     return _ruli_srv_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
   }
 
