@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.74 2004/06/25 23:19:11 evertonm Exp $
+  $Id: ruli_srv.c,v 1.75 2004/08/25 19:04:23 evertonm Exp $
   */
 
 
@@ -202,7 +202,7 @@ static void *on_walk_answer(ruli_host_t *host_qry, void *qry_arg)
 
     /* Move addresses from addr query to SRV entry */
     for (i = 0; i < addr_list_size; ++i) {
-      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      ruli_addr_t *addr = (ruli_addr_t *) ruli_list_get(addr_list, i);
       if (ruli_list_push(&srv_entry->addr_list, addr)) {
         ruli_list_prune(&srv_entry->addr_list, 0);
         return walk_query_done(walk_qry, RULI_SRV_CODE_FALL_OTHER);
@@ -395,7 +395,7 @@ static void *on_fallback_answer(ruli_host_t *qry, void *qry_arg)
 
     /* Move addresses from addr query to SRV entry */
     for (i = 0; i < addr_list_size; ++i) {
-      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      ruli_addr_t *addr = (ruli_addr_t *) ruli_list_get(addr_list, i);
       if (ruli_list_push(&srv_entry->addr_list, addr)) {
         ruli_list_prune(&srv_entry->addr_list, 0);
         return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER, qry);
